require "import"
import "android.widget.*"
import "android.view.*"
import "android.app.*"
import "android.media.*"
import "java.io.*"
import "android.net.Uri"
import "android.content.Intent"
import "android.provider.MediaStore"
import "java.nio.ByteBuffer"
import "android.os.Environment"

local activity = activity
local selectedFile

function onCreate(savedInstanceState)
    activity.setTitle("Chuyển đổi video sang âm thanh")
    local layout = LinearLayout(activity)
    layout.setOrientation(1)
    layout.setPadding(16, 16, 16, 16)

    local selectFileButton = Button(activity)
    selectFileButton.setText("Chọn tập tin video")
    selectFileButton.setOnClickListener{
        onClick = function()
            selectVideoFile()
        end
    }
    layout.addView(selectFileButton)

    local fileNameTextView = TextView(activity)
    fileNameTextView.setText("không có tập tin nào được chọn")
    layout.addView(fileNameTextView)

    local convertButton = Button(activity)
    convertButton.setText("Chuyển đổi sang định dạng  MP3")
    convertButton.setOnClickListener{
        onClick = function()
            convertToMP3()
        end
    }
    layout.addView(convertButton)

    activity.setContentView(layout)

    function updateUI(fileName)
        fileNameTextView.setText("Đã chọn tập tin: " .. fileName)
    end
end

function selectVideoFile()
    local intent = Intent(Intent.ACTION_GET_CONTENT)
    intent.setType("video/*")
    activity.startActivityForResult(intent, 1)
end

function onActivityResult(requestCode, resultCode, data)
    if requestCode == 1 and resultCode == Activity.RESULT_OK then
        local uri = data.getData()
        selectedFile = getRealPathFromURI(uri)
        updateUI(selectedFile:match("([^/]+)$"))
    end
end

function getRealPathFromURI(uri)
    local cursor = activity.getContentResolver().query(uri, nil, nil, nil, nil)
    cursor.moveToFirst()
    local idx = cursor.getColumnIndex(MediaStore.Video.VideoColumns.DATA)
    local result = cursor.getString(idx)
    cursor.close()
    return result
end

function convertToMP3()
    if not selectedFile then
        Toast.makeText(activity, "Vui lòng chọn tập tin video", Toast.LENGTH_SHORT).show()
        return
    end

    local outputFileName = File(selectedFile).getName():gsub("%.%w+$", "") .. ".mp3"
    local outputPath = Environment.getExternalStorageDirectory().getAbsolutePath() .. "/" .. outputFileName

    local extractor = MediaExtractor()
    extractor.setDataSource(selectedFile)

    local trackCount = extractor.getTrackCount()
    local audioTrackIndex = -1

    for i = 0, trackCount - 1 do
        local format = extractor.getTrackFormat(i)
        local mime = format.getString(MediaFormat.KEY_MIME)
        if mime:find("audio/") then
            audioTrackIndex = i
            break
        end
    end

    if audioTrackIndex == -1 then
        Toast.makeText(activity, "Không có âm thanh nào trong video đã chọn", Toast.LENGTH_SHORT).show()
        return
    end

    extractor.selectTrack(audioTrackIndex)

    local format = extractor.getTrackFormat(audioTrackIndex)
    local maxBufferSize = format.getInteger(MediaFormat.KEY_MAX_INPUT_SIZE)
    local buffer = ByteBuffer.allocate(maxBufferSize)
    local bufferInfo = MediaCodec.BufferInfo()

    local muxer = MediaMuxer(outputPath, MediaMuxer.OutputFormat.MUXER_OUTPUT_MPEG_4)
    local trackIndex = muxer.addTrack(format)
    muxer.start()

    while true do
        local sampleSize = extractor.readSampleData(buffer, 0)
        if sampleSize < 0 then
            break
        end

        bufferInfo.size = sampleSize
        bufferInfo.offset = 0
        bufferInfo.flags = extractor.getSampleFlags()
        bufferInfo.presentationTimeUs = extractor.getSampleTime()

        muxer.writeSampleData(trackIndex, buffer, bufferInfo)
        extractor.advance()
    end

    muxer.stop()
    muxer.release()
    extractor.release()

    Toast.makeText(activity, "Chuyển đổi tập tin thành công, được lưu dưới dạng: " .. outputPath, Toast.LENGTH_LONG).show()
end